//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const FLOCK_PHOTO_API_MANAGER_CID               = Components.ID('{18e27f2e-c99c-4915-9907-1a9fc0780ed6}');

const nsISupports                   = Components.interfaces.nsISupports;
const nsIClassInfo                  = Components.interfaces.nsIClassInfo;
const nsIFactory                    = Components.interfaces.nsIFactory;
const nsIProperties                 = Components.interfaces.nsIProperties;
const nsILocalFile                  = Components.interfaces.nsILocalFile;
const nsIFile                       = Components.interfaces.nsIFile;
const nsIIOService                  = Components.interfaces.nsIIOService;
const nsIFileProtocolHandler        = Components.interfaces.nsIFileProtocolHandler;
const nsIPreferenceService          = Components.interfaces.nsIPrefBranch;
const flockIPhotoAPIManager         = Components.interfaces.flockIPhotoAPIManager;
const flockIPhotoAPI                = Components.interfaces.flockIPhotoAPI;
const flockIPhotoPeopleService      = Components.interfaces.flockIPhotoPeopleService;
const flockIPollingService          = Components.interfaces.flockIPollingService;
const flockIMigratable              = Components.interfaces.flockIMigratable;

const FLOCK_PHOTO_API_MANAGER_CONTRACTID        = '@flock.com/photo-api-manager;1?';
const FLOCK_PHOTO_CONTRACTID        = '@flock.com/photo;1';
const DIRECTORY_SERVICE_CONTRACTID  = '@mozilla.org/file/directory_service;1';
const LOCAL_FILE_CONTRACTID         = '@mozilla.org/file/local;1';
const PREFERENCES_CONTRACTID        = '@mozilla.org/preferences-service;1';
const IO_SERVICE_CONTRACTID         = '@mozilla.org/network/io-service;1';
const RDFCU_CONTRACTID              = '@mozilla.org/rdf/container-utils;1';

const RDF_NS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
const FLOCK_NS = "http://flock.com/rdf#";
const NC_NS = "http://home.netscape.com/NC-rdf#";
const NSCP_NS = 'http://home.netscape.com/NC-rdf#';

/* for Cardinal to Danphe migration */
const OLD_PEOPLE_PHOTO_RDF_FILE = 'flock_people_photo.rdf';
const OLD_PEOPLE_PHOTO_RDF_FILE_RELIC = 'flock_people_photo_old.rdf';
const LIST_ACCOUNTS_ROOT = 'urn:flock:people:photo:lists:accounts';
const RDFS = Components.classes['@mozilla.org/rdf/rdf-service;1']
           .getService (Components.interfaces.nsIRDFService);
const IOS = Components.classes[IO_SERVICE_CONTRACTID]
          .getService(Components.interfaces.nsIIOService);
const RDFCU = Components.classes[RDFCU_CONTRACTID]
            .getService(Components.interfaces.nsIRDFContainerUtils);

function flockPhotoAPIManager() {
    this.mName2ServiceMap = {};
    this.mService2NameMap = {};
    this.mPrefService = Components.classes[PREFERENCES_CONTRACTID].getService(nsIPreferenceService);

    this.mLogger = Components.classes["@flock.com/logger;1"].createInstance(Components.interfaces.flockILogger);
    this.mLogger.init("photo");
    this.mLogger.info("starting up...");
    
    var catmgr = Components.classes["@mozilla.org/categorymanager;1"]
        .getService (Components.interfaces.nsICategoryManager);
    var enum = catmgr.enumerateCategory("flockIPhotoAPI");
    while(enum.hasMoreElements()) {
        try {
            var obj = enum.getNext();
            var supportsString = obj.QueryInterface(Components.interfaces.nsISupportsCString);
            var shortName = supportsString.toString();
            var cid = catmgr.getCategoryEntry("flockIPhotoAPI", shortName); 
            var svc = Components.classes[cid].getService(Components.interfaces.flockIPhotoAPI);
            this.registerAPI(svc, shortName);
        }
        catch(e) {
        }
    }
 

    var inst = this;
    var timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
    var timerFunc = {
        notify: function(aTimer) {
            inst.updateStates();
        },
    }
    timer.initWithCallback(timerFunc, 5 * 1000, 1);
}

//flockIMigratable

flockPhotoAPIManager.prototype.__defineGetter__('shortname', function() {
  return "Photo Accounts";
});

flockPhotoAPIManager.prototype.needsMigration =
function needsMigration(oldVersion) {
  var oldPeoplePhotoFile = Components.classes["@mozilla.org/file/directory_service;1"]
                         .getService(Components.interfaces.nsIProperties)
                         .get("ProfD", Components.interfaces.nsIFile);
  oldPeoplePhotoFile.append(OLD_PEOPLE_PHOTO_RDF_FILE);

  if (oldPeoplePhotoFile.exists()) {
    return true;
  }
  return false;  
}

flockPhotoAPIManager.prototype.startMigration =
function startMigration(aFlockMigrationProgressListener) {
 
    // ****************************************
    // photo migration from Cardinal to Danphe
    // ****************************************

    var oldPeoplePhotoFile = Components.classes["@mozilla.org/file/directory_service;1"]
                           .getService(Components.interfaces.nsIProperties)
                           .get("ProfD", Components.interfaces.nsIFile);
    oldPeoplePhotoFile.append(OLD_PEOPLE_PHOTO_RDF_FILE);

    var ctxt = {
        listener: aFlockMigrationProgressListener,
        oldPeoplePhotoFile: oldPeoplePhotoFile
    };

    if (oldPeoplePhotoFile.exists())
      ctxt.listener.onUpdate(0, 'Migrating photo people');

    return { wrappedJSObject: ctxt };
}

flockPhotoAPIManager.prototype.finishMigration =
function finishMigration(ctxtWrapper) {
}

flockPhotoAPIManager.prototype.doMigrationWork =
function doMigrationWork(ctxtWrapper) {
    var ctxt = ctxtWrapper.wrappedJSObject;

    if (!ctxt.oldPeoplePhotoFile.exists())
        return false;

    if (!ctxt.photoMigrator)
        ctxt.photoMigrator = this._migratePhotoPeople(ctxt);
    if (ctxt.photoMigrator.next())
        ctxt.photoMigrator = null;

    return Boolean(ctxt.photoMigrator);
}

flockPhotoAPIManager.prototype._migratePhotoPeople =
function _migratePhotoPeople(ctxt)
{
  // Load old datasource
  var oldPhotoPeepsDS = RDFS.GetDataSourceBlocking(IOS.newFileURI(ctxt.oldPeoplePhotoFile).spec);
  var acctsRoot = RDFS.GetResource(LIST_ACCOUNTS_ROOT);
  var streamsRoot = RDFS.GetResource("urn:flock:people:photo:lists:watched");
  var getValue = function (aSource, aArcName) {
    var target = oldPhotoPeepsDS.GetTarget(aSource, RDFS.GetResource(FLOCK_NS+aArcName), true);
    target instanceof Components.interfaces.nsIRDFLiteral;
    return target.Value;
  };

  // Migrate accounts
  var accounts = RDFCU.MakeSeq(oldPhotoPeepsDS, acctsRoot).GetElements(); 
  while (accounts && accounts.hasMoreElements()) {
    var account = accounts.getNext();
    account.QueryInterface(Components.interfaces.nsIRDFResource);
    var username = getValue(account, "username");
    var apiShortName = getValue(account, "apiShortName");
    var id = getValue(account, "id");
    var api = this.getAPIFromShortname(apiShortName);
    api.migrateAccount(id, username);
  }

  // Load new datasource
  var _coop = Components.classes["@flock.com/singleton;1"]
                        .getService(Components.interfaces.flockISingleton)
                        .getSingleton("chrome://browser/content/flock/common/load-faves-coop.js")
                        .wrappedJSObject;
  var mediaFavesURN = "urn:media:favorites";
  var mediaFaves = _coop.get(mediaFavesURN);
  if (!mediaFaves) {
    mediaFaves = new _coop.Folder(mediaFavesURN);
    _coop.favorites_root.children.add(mediaFaves);
  }

  // Migrate favorite photo streams
  var streams = RDFCU.MakeSeq(oldPhotoPeepsDS, streamsRoot).GetElements();
  while (streams && streams.hasMoreElements()) {
    var stream = streams.getNext();
    stream.QueryInterface(Components.interfaces.nsIRDFResource);
    var apiShortName = getValue(stream, "apiShortName");
    var id = getValue(stream, "id");
    var mediaqueryURN = "urn:media:favorites:"+apiShortName+":user:"+id;
    var mediaquery = new _coop.MediaQuery(
      mediaqueryURN,
      {
        serviceId: FLOCK_PHOTO_API_MANAGER_CONTRACTID,
        service: apiShortName,
        favicon: this.getAPIFromShortname(apiShortName).iconUrl,
        query: "user:"+id,
        name: getValue(stream, "username"),
        isPollable: true
      }
    );
    mediaFaves.children.add(mediaquery);
  }

  // rename the old people photo file
  ctxt.oldPeoplePhotoFile.moveTo(null, OLD_PEOPLE_PHOTO_RDF_FILE_RELIC);
  yield true;
}

//////////////////////////////////////////////////////////////////////////////
// Implementation
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////

flockPhotoAPIManager.prototype.mStates = {};
flockPhotoAPIManager.prototype.mListeners = new Array();

flockPhotoAPIManager.prototype.addListener = function(aListener) {
    if (aListener == null) {
        throw  "Listener is null!!";
    }
    this.mListeners.push(aListener);
}

flockPhotoAPIManager.prototype.updateStates = function() {
    for(var p in this.mStates) {
        var shortName = p;
        var state = this.mStates[p];
        var api = this.getAPIFromShortname(p);
        var newState = api.authState;
        if(newState != state) {
            this.notify(api);
        }
        this.mStates[p] = newState;
    }
}

flockPhotoAPIManager.prototype.notify = function(aService) {
    for(var i=0;i<this.mListeners.length;++i) {
            this.mListeners[i].onAPIStateChange(aService);
    }
}

flockPhotoAPIManager.prototype.doLogin = function(aAccountURN) {
    // TODO: MAYBE FIXME (CDC):
    // I changed the param for this function from aShortname to aAccountURN
    // ... but I don't think this is actually called from anywhere so for now
    // I'm gonna just throw NOT_IMPLEMENTED instead of fixing it to handle the
    // new parameter.  If it breaks things, I owe the first person who notices
    // it a bubble tea.
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    var inst = this;
    var myListener = {
        onAuth: function() {
            inst.notify(service);
            inst.setDefaultService(service.shortName);
        },
        onError: function(aError) {
           for (var i=0; i < inst.mListeners.length; i++) {
               aError.QueryInterface(Components.interfaces.flockIError);
               try {
                    var theError = {};
                    theError.errorCode = aError.errorCode;
                    theError.serviceErrorCode = aError.serviceErrorCode;
                    theError.serviceErrorString = aError.serviceErrorString;
                    theError.errorString = aError.errorString;
                    inst.mListeners[i].onError(theError);
               } catch (ex) {
                    this.mLogger.error(ex);
               }
            }
            
        },
    }
    var service = this.getAPIFromShortname(aShortname);
    service.login(null, myListener);
    this.notify(service);
}

flockPhotoAPIManager.prototype.doLogout = function(aShortname) {
    var service = this.getAPIFromShortname(aShortname);
    service.logout();

    // TODO - maybe change the default service to something that is enabled ???
    
    this.notify(service);
}


flockPhotoAPIManager.prototype.removeListener = function(aListener) {
    for(var i=0;i<this.mListeners.length;++i) {
        if(aListener==this.mListeners[i]) {
            this.mListeners.splice(i,1);
            break;
        }
    }
}

flockPhotoAPIManager.prototype.registerAPI = function(aService, aShortname) {
    
    if (this.mPrefService.getPrefType('flock.photo.service.' + aShortname + '.isDisabled')) {
        if (this.mPrefService.getBoolPref('flock.photo.service.' + aShortname + '.isDisabled')) {
            return true;
        }
        this.mPrefService.setBoolPref('flock.photo.service.' + aShortname + '.isDisabled', false);
    }        
    this.mName2ServiceMap[aShortname] = aService;
    this.mService2NameMap[aService] = aShortname;
    this.mStates[aShortname] = aService.authState;
}

flockPhotoAPIManager.prototype.isPreferredService = function (aShortname) {
    return this.mPreferredServices[aShortname];
}

flockPhotoAPIManager.prototype.getAPIFromShortname = function(aShortname) {
  // JMC - Special case, me!
    if (aShortname == 'preview') return this;
    return this.mName2ServiceMap[aShortname];
}


function loadLibraryFromSpec(aSpec)
{
  var loader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                         .getService(Components.interfaces.mozIJSSubScriptLoader);
  loader.loadSubScript(aSpec);
}

loadLibraryFromSpec("chrome://browser/content/flock/common/aggregate.js");

flockPhotoAPIManager.prototype.supportsSearch = function (aQuery) {
  return false;
}

flockPhotoAPIManager.prototype.search = function (aListener, aQuery, aCount, aPage) {
  // JMC - PQ of all mediaquery objects with new stuff
  var faves_coop = Components.classes['@flock.com/singleton;1']
                             .getService(Components.interfaces.flockISingleton)
                             .getSingleton('chrome://browser/content/flock/common/load-faves-coop.js')
                             .wrappedJSObject;
                             
  // JMC - If it's a page =1 request, perform the search
  // Otherwise, return the next batch from the cached results
  if (aPage > 1) {
    aListener.onSearchResult(this.myUnseenEnum);
  } else {                            
    var mediaQueries = faves_coop.MediaQuery.find({hasUnseenItems: true});
    var streams = [];
    for (x in mediaQueries) {
      var mediaquery = mediaQueries[x];
      streams.push(mediaquery.children.enumerate());
    }
    var aggUnSeen = filterStream( aggregatePhotoStreams( streams ), function(item) {
         return item.unseen;
       });
    var inst = this;   
    this.myUnseenEnum = {
      hasMoreElements : function() {
          return this.mEnum.hasMoreElements();
      },
      getNext : function() {
        var child = this.mEnum.getNext();
        // JMC - Get the real flockIPhoto from the coop obj.
        var svcName = child.getParent().service;
        return inst.getAPIFromShortname(svcName).getPhotoFromRDFNode(child.id());
      },
    };
    this.myUnseenEnum.mEnum = aggUnSeen;   
    aListener.onSearchResult(this.myUnseenEnum);
  }
}

flockPhotoAPIManager.prototype.__defineGetter__('services', function () { 
    var ar = new Array();
    for(var p in this.mName2ServiceMap) {
        ar.push(this.mName2ServiceMap[p]);
    }
    var rval = {
        getNext: function() {
            var rval = ar.shift();
            return rval;
        },
        hasMoreElements: function() {
            return (ar.length>0);
        },
    }
    return rval;
})

flockPhotoAPIManager.prototype.getPreferredServices = function() {
    try {
        var pref = this.mPrefService.getCharPref("flock.photo.preferredServices");
        var ar = pref.split(",");
        var rval = [];
        for(var i=0;i<ar.length;++i) {
            if(ar[i].length) rval.push(ar[i]); 
        }
        return rval;
    } catch(e) {
        //debug(e);
        debug("getPreferredServices: No preferred services found, returning an empty set\n");
        return [];
    }
}

flockPhotoAPIManager.prototype.setPreferredServices = function(aArray) {
    var val = aArray.join(",");
    var pref = this.mPrefService.setCharPref("flock.photo.preferredServices", val);
}

flockPhotoAPIManager.prototype.getDefaultService = function() {
    try {
        var pref = this.mPrefService.getCharPref("flock.photo.lastService");
        if (pref && pref.length)
            return this.getAPIFromShortname(pref);
    } catch(e) {
        return null;
    }
}

flockPhotoAPIManager.prototype.setDefaultService = function(aShortname) {
    try {
        this.mPrefService.setCharPref("flock.photo.lastService", aShortname);
    } catch(e) {
        debug(e);
    }
}

flockPhotoAPIManager.prototype.__defineGetter__('preferredServices', function () { 
try {
    var inst = this;
    var ar = this.getPreferredServices();
    var rval = {
        getNext: function() {
            var shortName = ar.shift();
            var api = inst.getAPIFromShortname(shortName);
            return api;
        },
        hasMoreElements: function() {
            return (ar.length>0);
        },
    }
    return rval;
    } catch(e) {
        debug(e);
    }
})


flockPhotoAPIManager.prototype.__defineGetter__("hasNewMedia", function () {
  var faves_coop = Components.classes['@flock.com/singleton;1']
                             .getService(Components.interfaces.flockISingleton)
                             .getSingleton('chrome://browser/content/flock/common/load-faves-coop.js')
                             .wrappedJSObject;
  var mediaFavsFolder = faves_coop.get('urn:media:favorites');
  var favItems = mediaFavsFolder.children.enumerate();
  if (favItems) {
    while (favItems.hasMoreElements()) {
       var favItem = favItems.getNext();
       if (favItem.hasUnseenItems) return true;
    }
  } else {
    return false;
  }
  return false;
})

flockPhotoAPIManager.prototype.processPhotoStream = function
photoAPI_processPhotoStream(aUrn, aEnum) {
  var faves_coop = Components.classes['@flock.com/singleton;1']
                              .getService(Components.interfaces.flockISingleton)
                              .getSingleton('chrome://browser/content/flock/common/load-faves-coop.js')
                              .wrappedJSObject;
                              
  var myStream = faves_coop.get(aUrn);
  if (!myStream) {
   return; 
  }  
  
  var biggestSeq = myStream.latestSeq;
  while (aEnum.hasMoreElements()) {
   var child = aEnum.getNext();
   child.QueryInterface(Components.interfaces.flockIPhoto);
   if (child.id > biggestSeq) biggestSeq = child.id;

   photo = child;
   var seq = photo.id;
   var urn = 'urn:flock:item:photo:'+myStream.service+':'+seq;

   var photoFav;
   photoFav = faves_coop.get(urn);
   var alreadyIn = (photoFav != null);
   var bumpPhotoObj = {};
   if (this.shouldAddPhoto(myStream, photo, bumpPhotoObj)) {
     var bumpPhoto = bumpPhotoObj.photo;
      // if it already exists, we're done
     if (!photoFav)  {
       photoFav = new faves_coop.RichPhoto(urn, {
        datevalue: new Date(parseInt(photo.uploadDate)),
        URL: photo.webPageUrl,
        thumbnail: photo.thumbnail,
        is_public: photo.is_public,
        is_video: photo.is_video,
        midSizePhoto: photo.midSizePhoto,
        name: photo.title,
        username: photo.username,
        userid: photo.userid,
        photoid: photo.id,
        largeSizePhoto: photo.largeSizePhoto,
       });
     }
    
    if (bumpPhoto) {
      myStream.children.insertAt(photoFav,myStream.children.indexOf(bumpPhoto));
      myStream.children.remove(bumpPhoto);
    }  else {
      myStream.children.addOnce(photoFav);
    }  
    if (!myStream.firstRefresh && !alreadyIn) { 
      photoFav.unseen = true;
    }
   }
  }
  if (biggestSeq > myStream.latestSeq) {
   myStream.latestSeq = biggestSeq;
   if (!myStream.firstRefresh) {
     myStream.hasUnseenItems = true;
     myStream.getParent().hasUnseenItems = true;
   }
  }
  myStream.firstRefresh = false;                         
}

// JMC - Returns null if it's not new enough
flockPhotoAPIManager.prototype.shouldAddPhoto = function
photoAPI_shouldAddPhoto(aStream, aPhoto, aReplacePhotoObj) {
  var numberOfPhotos = 5;
  var aEnum = aStream.children.enumerate();
  var i = 0;
  while (aEnum.hasMoreElements() && i < numberOfPhotos) {
    var aCurrentPhoto = aEnum.getNext();
    if (aCurrentPhoto.photoid == aPhoto.id) return false;
    if (aPhoto.id > aCurrentPhoto.photoid) {
      aReplacePhotoObj.photo =  aCurrentPhoto;
      return true;
    }
    i++;
  }
  if (i < numberOfPhotos) return true;
  return false;
}

flockPhotoAPIManager.prototype.refresh = function(aUrn, aListener) {
  // TODO XXX: This is brittle
  // Ought to be able to refresh non-user queries
  // Such as search terms, and other channels
  // Also, the tightly coupled mess with People Notifications 
  // Needs to go away

  // In the meantime, I'm gonna make an ugly special-case for
  // the mediaQuery object, which will become the general case later
  var faves_coop = Components.classes['@flock.com/singleton;1']
                             .getService(Components.interfaces.flockISingleton)
                             .getSingleton('chrome://browser/content/flock/common/load-faves-coop.js')
                             .wrappedJSObject;
  var photoAPIManager = this;
  var listener = function(aStream, aSearchListener) {
    this.mStream = aStream;
    this.mListener = aSearchListener;
  }
  
  var myStream = faves_coop.get(aUrn);
  if (!myStream) {
   aListener.onError();
   return; 
  }


  if (myStream.isInstanceOf(faves_coop.MediaQuery)) {

   listener.prototype.onSearchResult = function(aEnum) {
     photoAPIManager.processPhotoStream(aUrn, aEnum);
     this.mListener.onResult(null, null);
   };

    listener.prototype.onError = function(aError) {
        photoAPIManager.mLogger.error(aError);
        if(this.mListener) this.mListener.onError(aError);
    };

    var theApi = this.getAPIFromShortname(myStream.service);
    if (!theApi) {
      aListener.onResult(null, null);
      return; 
    }
    // (aListener, aQuery, aCount, aPage)
    theApi.search(new listener(myStream, aListener), myStream.query, null, null); // , 10, 1, null);
    return;
  }

  listener.prototype.onSearchResult = function(aEnumerator) {
    var newCount = 0;
    var notifURL;
    // JMC - Flip this around for performance fix
    var stuff = [];
    while(aEnumerator.hasMoreElements()) {
      var child = aEnumerator.getNext();
      stuff.push(child);
    }
    
    var lister = this;
    function myWorker(shouldYield) {
      
      while(stuff.length) {
        var photo  = stuff.shift();
        photo.QueryInterface(Components.interfaces.flockIPhoto);
        var seq = photo.id;
        photoAPIManager.mLogger.info("Found the photo:" + seq);
        photoAPIManager.mLogger.info("uploadDate of:" + photo.uploadDate);
        var urn = 'urn:flock:item:photo:'+lister.mStream.apiType+':'+seq;
        
        var photoFav;
        photoFav = faves_coop.get(urn);
        if (!photoFav) {
          photoFav = new faves_coop.Photo(urn, {
            datevalue: new Date(photo.uploadDate),
            URL: photo.webPageUrl,
            midSizePhoto: photo.midSizePhoto,
          });
        }
        
        if (lister.mStream.children.indexOf(photoFav) < 0) {
          lister.mStream.children.addOnce(photoFav);
          photoFav.unseen = true;
          newCount++;
          if (!notifURL)
            notifURL = photo.webPageUrl;
        }
        if (shouldYield()) yield;
      }
    
      lister.mListener.onResult(null, null);
    }
    
    schedule (0.4, 40, myWorker);
    
  };

  listener.prototype.onError = function(aError) {
      photoAPIManager.mLogger.error(aError);
      if(this.mListener) this.mListener.onError(aError);
  };


/*  var e = myStream.children.enumerate ();
  while (e.hasMoreElements ()) {
    var child = e.getNext();
    myStream.children.remove(child); // TODO: also delete the resource itself?
  } */

  var theApi = this.getAPIFromShortname(myStream.apiType);
  if (!theApi) {
   aListener.onResult(null, null);
   return; 
  }
  // (aListener, aQuery, aCount, aPage)
  theApi.search(new listener(myStream, aListener), "user:" + myStream.userid, null, null); // , 10, 1, null);
}


flockPhotoAPIManager.prototype.flags = nsIClassInfo.SINGLETON;
flockPhotoAPIManager.prototype.classDescription = "Flock Photo People Service";
flockPhotoAPIManager.prototype.getInterfaces = function (count) {
    var interfaceList = [Components.interfaces.flockISocialWebService, flockIPhotoAPIManager, flockIPollingService,
                         flockIMigratable, nsIClassInfo];
    count.value = interfaceList.length;
    return interfaceList;
}

flockPhotoAPIManager.prototype.getHelperForLanguage = function (count) {return null;}

// JMC - Making this a social photo service for the preview stream
flockPhotoAPIManager.prototype.QueryInterface =
function (iid) {
    if (!iid.equals(Components.interfaces.flockISocialWebService) &&
        !iid.equals(flockIPhotoAPIManager) && 
        !iid.equals(flockIPollingService) &&
        !iid.equals(flockIMigratable) &&
        !iid.equals(nsIClassInfo) &&
        !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
}

// Module implementation
var PhotoPeopleModule = new Object();

PhotoPeopleModule.registerSelf =
function (compMgr, fileSpec, location, type)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(FLOCK_PHOTO_API_MANAGER_CID, 
                                    "Flock Photo API Manager JS Component",
                                    FLOCK_PHOTO_API_MANAGER_CONTRACTID, 
                                    fileSpec, 
                                    location,
                                    type);
    var catman = Components.classes['@mozilla.org/categorymanager;1']
      .getService(Components.interfaces.nsICategoryManager);
     
    catman.addCategoryEntry('flockMigratable', 'photoservice', FLOCK_PHOTO_API_MANAGER_CONTRACTID, true, true);
}

PhotoPeopleModule.getClassObject =
function (compMgr, cid, iid) {
    if (!cid.equals(FLOCK_PHOTO_API_MANAGER_CID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return PhotoAPIManagerFactory;
}

PhotoPeopleModule.canUnload =
function(compMgr)
{
    return true;
}
    
/* factory object */
var PhotoAPIManagerFactory = new Object();

PhotoAPIManagerFactory.createInstance =
function (outer, iid) {
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

    return (new flockPhotoAPIManager()).QueryInterface(iid);
}

/* entrypoint */
function NSGetModule(compMgr, fileSpec) {
    return PhotoPeopleModule;
}



// a utility for async scheduling:

// cheap scheduler for javascript tasks

// aSlice is the maximum timeslice in seconds. 0.5 or less is a good value

// aPercent is the percent of wall-clock time that should be spent executing
// the task - eg: 20 = one fifth of the time

// aTask is defined to be a generator taking an function that evaluates
// to true if it should yield like:
// function task (should_yield) {
//   while (some_condition) {
//     do some work...
//     if (should_yield()) yield;
//   }
// }
// NOTE: should_yield() is fairly cheap but in a tight inner loop it might
//       make sense to call it every N cycles

var timers = [];
function schedule (aSlice, aPercent, aTask) {
  // convert percent to ratio
  var ratio = aPercent / 100.0;

  // convert slice to milliseconds
  var slice = aSlice * 1000.0;

  function milliseconds () { return (new Date ()).getTime (); }
  var wall_start = milliseconds ();
  var process_time = 0;

   
  var process_start = 0;
  function should_yield () {
    var now = milliseconds ();
    var so_far = now - process_start;
    var wall_time = now - wall_start;
    return so_far >= slice || ((wall_time * ratio) < (process_time + so_far));
  }

  var generator = aTask (should_yield);

  var timer = Components.classes['@mozilla.org/timer;1']
    .createInstance (Components.interfaces.nsITimer);
  var callback = {
    notify: function () {
      var wall_time = milliseconds () - wall_start;

      if ((wall_time * ratio) < process_time) {
        // if we've spent more time than we're supposed to
        // don't run this cycle
      }

      process_start = milliseconds ();
      try {
        generator.next ();
      } catch (err if err instanceof StopIteration) {
        // the task is complete
        timer.cancel ();
        for (var i=0; i<timers.length; i++) { 
          if (timers[i] == timer) {
            timers.splice (i,1);
          }
        }
        delete timer;
      }
      process_time += (milliseconds () - process_start);
    }
  }

  timer.initWithCallback (callback, Math.round (slice/ratio),
      Components.interfaces.nsITimer.TYPE_REPEATING_SLACK)
  timers.push (timer);
}

